
import Debug from 'debug'
import Emitter from 'events'
import immediate from 'p-immediate'

/**
 * Debug.
 *
 * @type {Function}
 */

const debug = Debug('nexus:pubsub-memory')

/**
 * The default, memory-only pubsub adapter.
 *
 * @type {MemoryPubsub}
 */

class MemoryPubsub extends Emitter {

  /**
   * Subscribe to a document channel by `id`.
   *
   * @param {String} id
   */

  async subscribe(id) {
    debug('subscribe', { id })
  }

  /**
   * Unsubscribe from a document channel by `id`.
   *
   * @param {String} id
   */

  async unsubscribe(id) {
    debug('unsubscribe', { id })
  }

  /**
   * Publish `message` to a document channel by `id`.
   *
   * @param {String} id
   * @param {Object} message
   */

  async publish(id, message) {
    debug('publish', { id, message })
    await immediate()
    this.emit('publish', id, message)
  }

}

/**
 * Export.
 *
 * @type {MemoryPubsub}
 */

export default MemoryPubsub
