
import { Duplex } from 'stream'

/**
 * An in-memory transport adapter.
 *
 * @type {MemoryTransport}
 */

class MemoryTransport {

  /**
   * Constructor.
   */

  constructor() {
    const client = new Duplex({ objectMode: true })
    const server = new Duplex({ objectMode: true })

    client._read = () => {}
    client._write = (data, encoding, next) => {
      setImmediate(() => {
        server.push(data)
        next()
      })
    }

    server._read = () => {}
    server._write = (data, encoding, next) => {
      setImmediate(() => {
        client.push(data)
        next()
      })
    }

    this.client = client
    this.server = server
  }

}

/**
 * Export.
 *
 * @type {Object}
 */

export default MemoryTransport
