
import template from 'lodash/template'

import ERRORS from '../constants/errors'

/**
 * Nexus error.
 *
 * @type {NexusError}
 */

class NexusError extends Error {

  /**
   * Create a Nexus error by `code` (or message) with `data`.
   *
   * @type {String} code|message
   * @param {Object} data
   */

  constructor(code, data = {}) {
    const obj = ERRORS[code]
    let message

    if (!obj) {
      message = `An unrecognized Nexus error was thrown! The code "${code}" is not a valid Nexus error code.`
    }

    else if (obj.message == null) {
      message = `An invalid Nexus error was defined! The "${code}" error has no message definition.`
    }

    else if (obj.data == null) {
      message = `An invalid Nexus error was defined! The "${code}" error has no property definition.`
    }

    else {
      const fn = template(obj.message, { interpolate: /{([\s\S]+?)}/g })
      message = `${code} - ${fn(data)}`

      for (const key of obj.data) {
        if (!(key in data)) {
          message = `An invalid Nexus error was thrown! The \`${key}\` error property was not provided when a "${code}" error was thrown.`
        }
      }

      for (const key in data) {
        if (!obj.data.includes(key)) {
          message = `An invalid Nexus error was thrown! The \`${key}\` error property is not one of the recognized properties of "${code}" errors.`
        }
      }
    }

    super(message)
    this.name = 'NexusError'
    this.code = code
    this.data = data
  }

}

/**
 * Export.
 *
 * @type {NexusError}
 */

export default NexusError
