
import JSON0 from 'ot-json0' // eslint-disable-line import/no-extraneous-dependencies
import assert from 'assert'
import delay from 'delay' // eslint-disable-line import/no-extraneous-dependencies
import fs from 'fs'
import { basename, extname, resolve } from 'path'

import Nexus, { MemoryStore } from '../..'

/**
 * Types.
 */

const types = [
  JSON0.type,
]

/**
 * Tests.
 */

describe('ot', () => {
  const testsDir = resolve(__dirname, 'fixtures')
  const tests = fs.readdirSync(testsDir).filter(t => t[0] !== '.').map(t => basename(t, extname(t)))

  for (const test of tests) {
    it(test, async () => {
      const module = require(resolve(testsDir, test))
      const { store, run, expected } = module
      const memoryStore = new MemoryStore(store)
      const nexus = new Nexus({ types, store: memoryStore })
      const client = await nexus.connect()
      const other = await nexus.connect()
      const doc = await run(client, other)

      await client.settle()
      await other.settle()

      // Wait for both clients to receive any pending messages from the other
      // one after they've settled.
      await delay(10)

      const result = {
        type: doc.type.name,
        version: doc.version,
        data: doc.value,
      }

      assert.deepEqual(result, expected)
    })
  }
})
